"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.query = exports.put = exports.update = exports.timestampCondition = exports.updateExpression = void 0;

var _highland = _interopRequireDefault(require("highland"));

var _merge = _interopRequireDefault(require("lodash/merge"));

var _memoryCache = _interopRequireDefault(require("memory-cache"));

var _dynamodb = _interopRequireDefault(require("../connectors/dynamodb"));

var _faults = require("./faults");

var _print = require("./print");

const updateExpression = Item => ({
  ExpressionAttributeNames: Object.keys(Item).map(attrName => ({
    [`#${attrName}`]: attrName
  })).reduce(_merge.default, {}),
  ExpressionAttributeValues: Object.keys(Item).map(attrName => ({
    [`:${attrName}`]: Item[attrName]
  })).reduce(_merge.default, {}),
  UpdateExpression: `SET ${Object.keys(Item).map(attrName => `#${attrName} = :${attrName}`).join(', ')}`,
  ReturnValues: 'ALL_NEW'
});

exports.updateExpression = updateExpression;

const timestampCondition = (fieldName = 'timestamp') => ({
  ConditionExpression: `attribute_not_exists(#${fieldName}) OR #${fieldName} < :${fieldName}`
});

exports.timestampCondition = timestampCondition;

const update = ({
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.ENTITY_TABLE_NAME,
  updateRequestField = 'updateRequest',
  parallel = Number(process.env.UPDATE_PARALLEL) || Number(process.env.PARALLEL) || 4
} = {}) => {
  const connector = new _dynamodb.default({
    debug,
    tableName
  });

  const invoke = uow => {
    const p = connector.update(uow[updateRequestField]).then(updateResponse => ({ ...uow,
      updateResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(invoke).parallel(parallel);
};

exports.update = update;

const put = ({
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.EVENT_TABLE_NAME,
  putRequestField = 'putRequest',
  parallel = Number(process.env.UPDATE_PARALLEL) || Number(process.env.PARALLEL) || 4
} = {}) => {
  const connector = new _dynamodb.default({
    debug,
    tableName
  });

  const invoke = uow => {
    const p = connector.put(uow[putRequestField]).then(putResponse => ({ ...uow,
      putResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(invoke).parallel(parallel);
};

exports.put = put;

const query = (
/* istanbul ignore next */
{
  debug = (0, _print.debug)('dynamodb'),
  tableName = process.env.EVENT_TABLE_NAME,
  queryRequestField = 'queryRequest',
  queryResponseField = 'queryResponse',
  parallel = Number(process.env.QUERY_PARALLEL) || Number(process.env.PARALLEL) || 4
} = {}) => {
  const connector = new _dynamodb.default({
    debug,
    tableName
  });

  const invoke = uow => {
    if (!uow[queryRequestField]) return (0, _highland.default)(Promise.resolve(uow));
    const req = JSON.stringify(uow[queryRequestField]);

    const cached = _memoryCache.default.get(req);

    const p = (cached ? Promise.resolve(cached) : connector.query(uow[queryRequestField]).then(queryResponse => {
      _memoryCache.default.put(req, queryResponse);

      return queryResponse;
    })).then(queryResponse => ({ ...uow,
      [queryResponseField]: queryResponse
    })).catch((0, _faults.rejectWithFault)(uow));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map(invoke).parallel(parallel);
};

exports.query = query;