"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.publishToEventBridge = void 0;

var _highland = _interopRequireDefault(require("highland"));

var _eventbridge = _interopRequireDefault(require("../connectors/eventbridge"));

var _batch = require("./batch");

var _faults = require("./faults");

var _print = require("./print");

var _tags = require("./tags");

const publishToEventBridge = ({
  // eslint-disable-line import/prefer-default-export
  debug = (0, _print.debug)('eventbridge'),
  busName = process.env.BUS_NAME || 'undefined',
  source = 'custom',
  eventField = 'event',
  batchSize = Number(process.env.PUBLISH_BATCH_SIZE) || Number(process.env.BATCH_SIZE) || 10,
  parallel = Number(process.env.PUBLISH_PARALLEL) || Number(process.env.PARALLEL) || 8,
  handleErrors = true
} = {}) => {
  const connector = new _eventbridge.default({
    debug
  });

  const toInputParams = batchUow => ({ ...batchUow,
    inputParams: {
      Entries: batchUow.batch.map(uow => ({
        EventBusName: busName,
        Source: source,
        DetailType: uow[eventField].type,
        Detail: JSON.stringify(uow[eventField])
      }))
    }
  });

  const putEvents = batchUow => {
    const p = connector.putEvents(batchUow.inputParams).catch((0, _faults.rejectWithFault)(batchUow, !handleErrors)).then(handleFailedEntries(batchUow)).then(publishResponse => ({ ...batchUow,
      publishResponse
    }));
    return (0, _highland.default)(p); // wrap promise in a stream
  };

  return s => s.map((0, _tags.adornStandardTags)(eventField)).batch(batchSize).map(_batch.toBatchUow).map(toInputParams).map(putEvents).parallel(parallel).flatMap(_batch.unBatchUow); // for cleaner logging and testing
};

exports.publishToEventBridge = publishToEventBridge;

const handleFailedEntries = batchUow => publishResponse => {
  if (publishResponse.FailedEntryCount === 0) {
    return publishResponse;
  } else {
    const failed = publishResponse.Entries.reduce((a, c, i) => {
      if (c.ErrorCode) {
        return [...a, { ...batchUow.batch[i],
          inputParam: batchUow.inputParams.Entries[i],
          err: {
            code: c.ErrorCode,
            msg: c.ErrorMessage
          }
        }];
      } else {
        return a;
      }
    }, []);
    return (0, _faults.throwFault)({
      batch: failed
    })(new Error(`Event Bridge Failed Entry Count: ${publishResponse.FailedEntryCount}`));
  }
};